@@ -536,11 +536,11 @@ bool AppleIVAD::doUpdate( void )
     IODisplay::setParameter( fDisplayParams, gIODisplayBrightnessKey,
             fUserPrefs->brightness );
     IODisplay::setParameter( fDisplayParams, gIODisplayHorizontalPositionKey,
-            fUserPrefs->modeParams[fModeIndex].horizontalPosition ^ 0x7f );
+            fUserPrefs->modeParams[fModeIndex].horizontalPosition ^ 0x7F );
     IODisplay::setParameter( fDisplayParams, gIODisplayHorizontalSizeKey,
-            fUserPrefs->modeParams[fModeIndex].horizontalSize ^ 0x7f );
+            fUserPrefs->modeParams[fModeIndex].horizontalSize ^ 0x7F );
     IODisplay::setParameter( fDisplayParams, gIODisplayVerticalPositionKey,
-            fUserPrefs->modeParams[fModeIndex].verticalPosition ^ 0x7f );
+            fUserPrefs->modeParams[fModeIndex].verticalPosition ^ 0x7F );
     IODisplay::setParameter( fDisplayParams, gIODisplayVerticalSizeKey,
             fUserPrefs->modeParams[fModeIndex].verticalSize );
     IODisplay::setParameter( fDisplayParams, gIODisplayTrapezoidKey,
@@ -548,7 +548,7 @@ bool AppleIVAD::doUpdate( void )
     IODisplay::setParameter( fDisplayParams, gIODisplayPincushionKey,
             fUserPrefs->modeParams[fModeIndex].pincushion );
     IODisplay::setParameter( fDisplayParams, gIODisplayParallelogramKey,
-            fUserPrefs->parallelogram  ^ 0x7f);
+            fUserPrefs->parallelogram ^ 0x7F);
     IODisplay::setParameter( fDisplayParams, gIODisplayRotationKey,
             fUserPrefs->tilt);
     IODisplay::setParameter( fDisplayParams, gIODisplayParametersTheatreModeKey,
@@ -1134,6 +1134,16 @@ bool AppleIVAD2::setDisplay( IODisplay* display )
 	if ( !fFramebuffer )
 		return( false );
 
+	if ( fNotifier != NULL )
+		{
+		// #3188989
+		// This routine can get called each time the display configuration changes.  For example,
+		// if an external monitor is attached, then this routine would get called.  In those cases,
+		// we want to release the previous notifier before we add a new one.
+
+		fNotifier->release();
+		}
+
 	fNotifier = fFramebuffer->addFramebufferNotification( &AppleIVAD2::_framebufferEvent, this, NULL );
 
 	setupIVAD();
@@ -1180,13 +1190,13 @@ bool AppleIVAD2::doUpdate( void )
 
 	IODisplay::setParameter( fDisplayParams, gIODisplayContrastKey, fUserPrefs->contrast );
 	IODisplay::setParameter( fDisplayParams, gIODisplayBrightnessKey, fUserPrefs->brightness );
-	IODisplay::setParameter( fDisplayParams, gIODisplayHorizontalPositionKey, fCurrentModeParams->horizontalPosition ^ 0x7f );
-	IODisplay::setParameter( fDisplayParams, gIODisplayHorizontalSizeKey, fCurrentModeParams->horizontalSize ^ 0x7f );
-	IODisplay::setParameter( fDisplayParams, gIODisplayVerticalPositionKey, fCurrentModeParams->verticalPosition ^ 0x7f );
+	IODisplay::setParameter( fDisplayParams, gIODisplayHorizontalPositionKey, fCurrentModeParams->horizontalPosition ^ 0x7F );
+	IODisplay::setParameter( fDisplayParams, gIODisplayHorizontalSizeKey, fCurrentModeParams->horizontalSize ^ 0x7F );
+	IODisplay::setParameter( fDisplayParams, gIODisplayVerticalPositionKey, fCurrentModeParams->verticalPosition ^ 0x7F );
 	IODisplay::setParameter( fDisplayParams, gIODisplayVerticalSizeKey, fCurrentModeParams->verticalSize );
 	IODisplay::setParameter( fDisplayParams, gIODisplayTrapezoidKey, fCurrentModeParams->keystone );
 	IODisplay::setParameter( fDisplayParams, gIODisplayPincushionKey, fCurrentModeParams->pincushion );
-	IODisplay::setParameter( fDisplayParams, gIODisplayParallelogramKey, fUserPrefs->parallelogram  ^ 0x7f );
+	IODisplay::setParameter( fDisplayParams, gIODisplayParallelogramKey, fUserPrefs->parallelogram );
 	IODisplay::setParameter( fDisplayParams, gIODisplayRotationKey, fUserPrefs->tilt );
 	IODisplay::setParameter( fDisplayParams, gIODisplayParametersTheatreModeKey, fTheatreMode );
 
@@ -1471,7 +1481,6 @@ bool AppleIVAD2::setupIVAD( void )
 	// Try and get the saved geometry preferences.  If we cannot find it,
 	// then regenerate it from the EEPROM data.
 
-	// Do we need to release data?
 	if ( ( data = OSDynamicCast( OSData, fIVAD->getProperty( fIVADPrefsKey ) ) ) )
 		{
 		UnpackGeometryProperty( data );
